# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Constants used by package manager widget."""

import enum


class AppType(str, enum.Enum):
    """Options for the application types."""

    WEB = 'web_app'
    CONDA = 'conda_app'
    INSTALLABLE = 'installable'



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Launch applications utilities."""

import datetime
import itertools
import os
import subprocess  # nosec
import sys
import typing
from navigator_updater.config import LAUNCH_SCRIPTS_PATH, WIN


if WIN:
    import ctypes


DEVNULL: typing.Final[str] = 'nul' if (sys.platform == 'win32') else '/dev/null'
LOG_FILE: typing.Final[str] = '{package}-{level}-{index}.txt'


class CommandDetails(typing.TypedDict):
    """Dictionary with extra details about executed command."""

    args: str
    id: int  # pylint: disable=invalid-name
    cmd: str
    stdout: str
    stderr: str


class GetCommand(typing.Protocol):  # pylint: disable=too-few-public-methods
    """Common interface for all `get_command_*` functions."""

    def __call__(  # pylint: disable=too-many-arguments
            self,
            root_prefix: str,
            prefix: str,
            command: str,
            extra_arguments: typing.Iterable[typing.Any] = (),
            package_name: str = 'app',
            environment: typing.Optional[typing.Mapping[str, str]] = None,
            cwd: str = os.path.expanduser('~'),
            default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
    ) -> typing.Tuple[typing.Mapping[str, typing.Any], CommandDetails]:
        """
        Generate script to launch application and return path to it.

        :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
        :param prefix: Conda prefix, which should be active.
        :param command: Actual command to launch application.
        :param extra_arguments: Additional arguments to attach to command.
        :param cwd: Custom working directory to launch application in.
        :param package_name: Name of the conda package, or alias of the external application.
        :param environment: Custom environment to launch application in.
        :param default_scripts_path: Root directory to store launch scripts in.
        :return: Path to generated launch script file.
        """


def safe_unix(value: str) -> str:
    """Prepare argument which is safe to use in unix command line."""
    to_replace: typing.Final[typing.Set[str]] = {'"', '$', '\\', '`'}
    to_escape: typing.Final[typing.Set[str]] = {' ', '&', ';', '<', '>', '|'} | to_replace

    if set(value) & to_escape:
        character: str
        for character in to_replace:
            value = value.replace(character, f'\\{character}')

        value = f'"{value}"'

    return value


def safe_windows(value: str) -> str:
    """Prepare argument which is safe to use in windows command line."""
    to_escape: typing.Final[typing.Set[str]] = {' ', '%', '"', '&', ';', '<', '>', '|'}

    if set(value) & to_escape:
        value = value.replace('"', '""').replace('%', '"^%"')
        value = f'"{value}"'

    return value


def expand_environment(
        command: str,
        environment: typing.Mapping[str, typing.Any],
        default: typing.Any = None,
        recursive: bool = False,
) -> str:
    """
    Update command with environment variables.

    :param command: Original command to update with environment variable values.
    :param environment: Environment variables to inject into command.

                        :code:`None` values won't be expanded.
    :param default: Value to insert for all unknown variables in `command`.
    :param recursive: Allow recursive expand of the values.
    :return: String with expanded environment.
    """
    cursor: int = 0
    while True:
        start: int = command.find('${', cursor)
        if start < 0:
            break

        stop: int = command.find('}', start)
        environment_key: str = command[start + 2:stop]
        stop += 1

        environment_value: typing.Any = environment.get(environment_key, default)
        if environment_value is None:
            cursor = stop
            continue

        environment_value = str(environment_value)
        command = command[:start] + environment_value + command[stop:]
        cursor = start
        if not recursive:
            cursor += len(environment_value)

    return command


class RunningProcess:
    """
    Minimal description of a process launched from the Navigator.

    Common example of such process - anything started from the Home page tiles.

    :param package: Name of the Conda package, or alias of a launched application.
    :param process: :class:`~subprocess.Popen` instance, which is used to launch the application.
    :param stdout: Path to a file with captured `stdout`.
    :param stderr: Path to a file with captured `stderr`.
    """

    __slots__ = ('__birth', '__package', '__process', '__return_code', '__stderr', '__stdout')

    def __init__(
            self,
            package: str,
            process: subprocess.Popen,
            stdout: typing.Optional[str] = None,
            stderr: typing.Optional[str] = None,
    ) -> None:
        """Initialize new :class:`~RunningProcess` instance."""
        if stdout == DEVNULL:
            stdout = None
        if stderr == DEVNULL:
            stderr = None

        self.__package: typing.Final[str] = package
        self.__process: typing.Final[subprocess.Popen] = process
        self.__return_code: typing.Optional[int] = None
        self.__stdout: typing.Final[typing.Optional[str]] = stdout
        self.__stderr: typing.Final[typing.Optional[str]] = stderr
        self.__birth: typing.Final[datetime.datetime] = datetime.datetime.utcnow()

    @property
    def age(self) -> datetime.timedelta:
        """Current age of the application."""
        return datetime.datetime.utcnow() - self.__birth

    @property
    def package(self) -> str:  # noqa: D401
        """Name of the Conda package, or alias of a launched application."""
        return self.__package

    @property
    def pid(self) -> int:  # noqa: D401
        """PID of the process."""
        return self.__process.pid

    @property
    def return_code(self) -> typing.Optional[int]:  # noqa: D401
        """Current status of the application."""
        if self.__return_code is None:
            self.__return_code = self.__process.poll()
        return self.__return_code

    @property
    def stderr(self) -> typing.Optional[str]:  # noqa: D401
        """Content of the `stdout` log."""
        return self.__read(self.__stderr)

    @property
    def stdout(self) -> typing.Optional[str]:  # noqa: D401
        """Content of the `stdout` log."""
        return self.__read(self.__stdout)

    @staticmethod
    def __read(path: typing.Optional[str]) -> typing.Optional[str]:
        """Read content of the file."""
        if path is None:
            return None
        try:
            stream: typing.TextIO
            with open(path, 'rt') as stream:  # pylint: disable=unspecified-encoding
                return stream.read()
        except OSError:
            return None

    def cleanup(self) -> None:
        """Remove all temporary content."""
        path: str
        for path in typing.cast(typing.Iterable[str], filter(bool, [self.__stdout, self.__stderr])):
            try:
                os.remove(path)
            except OSError:
                pass


def get_scripts_path(
        root_prefix: str,
        prefix: str,
        default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> str:
    """
    Return path to directory, where all launch scripts should be placed.

    This directory can also be used for the application logs.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param default_scripts_path: Root directory to store launch scripts in.
    :return: Path to directory, where launch script should be placed.
    """
    result: str = os.path.abspath(default_scripts_path)

    root_prefix = os.path.abspath(root_prefix)
    prefix = os.path.abspath(prefix)
    if prefix != root_prefix:
        result = os.path.join(result, os.path.basename(prefix))

    os.makedirs(result, exist_ok=True)
    return result


def remove_package_logs(
        root_prefix: str,
        prefix: str,
        default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> None:
    """
    Try to remove output, error logs for launched applications.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param default_scripts_path: Root directory to store launch scripts in.
    """
    logs_root: str = get_scripts_path(
        root_prefix=root_prefix,
        prefix=prefix,
        default_scripts_path=default_scripts_path,
    )

    file_path: str
    for file_path in os.listdir(logs_root):
        if not file_path.endswith('.txt'):
            continue
        try:
            os.remove(os.path.join(logs_root, file_path))
        except OSError:
            pass


def get_package_logs(
    root_prefix: str,
    prefix: str,
    package_name: str = 'app',
    id_: typing.Optional[int] = None,
    default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> typing.Tuple[str, str, int]:
    """
    Return the package log names for launched applications.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param package_name: Name of the conda package, or alias of the external application.
    :param id_: Application session identifier.

                Used to create different names for different application launch sessions.
    :param default_scripts_path: Root directory to store launch scripts in.
    """
    logs_root: str = get_scripts_path(
        root_prefix=root_prefix,
        prefix=prefix,
        default_scripts_path=default_scripts_path,
    )

    stdout_log_path: str = LOG_FILE.format(package=package_name, level='out', index=id_)
    stderr_log_path: str = LOG_FILE.format(package=package_name, level='err', index=id_)
    if id_ is None:
        used: typing.Set[str] = set(os.listdir(logs_root))

        for id_ in itertools.count(start=1):  # pylint: disable=redefined-argument-from-local
            stdout_log_path = LOG_FILE.format(package=package_name, level='out', index=id_)
            stderr_log_path = LOG_FILE.format(package=package_name, level='err', index=id_)
            if (stdout_log_path not in used) and (stderr_log_path not in used):
                break

    if prefix and root_prefix:
        stdout_log_path = os.path.join(logs_root, stdout_log_path)
        stderr_log_path = os.path.join(logs_root, stderr_log_path)

    return stdout_log_path, stderr_log_path, id_


def create_app_run_script(  # pylint: disable=too-many-arguments
    root_prefix: str,
    prefix: str,
    command: str,
    extension: str,
    package_name: str = 'app',
    default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> str:
    """
    Create new application launching script file.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param command: Content of the script file.
    :param extension: Extension for the script file (`.bat`, `.sh`)
    :param package_name: Name of the conda package, or alias of the external application.
    :param default_scripts_path: Root directory to store launch scripts in.
    :return: Path to generated launch script file.
    """
    scripts_root: str = get_scripts_path(
        root_prefix=root_prefix,
        prefix=prefix,
        default_scripts_path=default_scripts_path,
    )

    encoding: str = 'utf-8'
    if WIN:
        encoding = f'cp{ctypes.cdll.kernel32.GetACP()}'

    stream: typing.TextIO
    file_path: str = os.path.join(scripts_root, f'{package_name}{extension}')
    with open(file_path, 'wt', encoding=encoding) as stream:
        stream.write(command)

    os.chmod(file_path, 0o755)  # nosec

    return file_path


def get_command_on_win(  # pylint: disable=too-many-arguments
        root_prefix: str,
        prefix: str,
        command: str,
        extra_arguments: typing.Iterable[typing.Any] = (),
        package_name: str = 'app',
        environment: typing.Optional[typing.Mapping[str, str]] = None,
        cwd: str = os.path.expanduser('~'),
        default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> typing.Tuple[typing.Mapping[str, typing.Any], 'CommandDetails']:
    """
    Generate script to launch application and return path to it.

    This function is optimized to run on Windows.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param command: Actual command to launch application.
    :param extra_arguments: Additional arguments to attach to command.
    :param cwd: Custom working directory to launch application in.
    :param package_name: Name of the conda package, or alias of the external application.
    :param environment: Custom environment to launch application in.
    :param default_scripts_path: Root directory to store launch scripts in.
    :return: Path to generated launch script file.
    """
    stdout_log_path: str
    stderr_log_path: str
    id_: int
    stdout_log_path, stderr_log_path, id_ = get_package_logs(
        root_prefix=root_prefix,
        prefix=prefix,
        package_name=package_name,
        default_scripts_path=default_scripts_path,
    )

    command = dict((
        (
            'start cmd.exe /K "${CONDA_ROOT_PREFIX}\\\\Scripts\\\\activate.bat" "${CONDA_PREFIX}"',
            'start cmd.exe \\K "${CONDA_ROOT_PREFIX}\\Scripts\\activate.bat" "${CONDA_PREFIX}"',
        ),
        (
            str(
                'start powershell.exe -ExecutionPolicy ByPass -NoExit -Command  '  # pylint: disable=implicit-str-concat
                '"& \'{CONDA_ROOT_PREFIX}\\\\shell\\\\condabin\\\\conda-hook.ps1\' ; conda activate \'{CONDA_PREFIX}\'"'
            ),
            str(
                'start powershell.exe -ExecutionPolicy ByPass -NoExit -Command '  # pylint: disable=implicit-str-concat
                '"& \'${CONDA_ROOT_PREFIX}\\shell\\condabin\\conda-hook.ps1\' ; conda activate \'${CONDA_PREFIX}\'"'
            ),
        )
    )).get(command, command)

    command = expand_environment(
        command=command,
        environment={
            'PREFIX': prefix,
            'CONDA_PREFIX': prefix,
            'CONDA_ROOT_PREFIX': root_prefix,
        },
    )
    activate: str = os.path.join(root_prefix, 'Scripts', 'activate')

    extra_argument: typing.Any
    for extra_argument in extra_arguments:
        command += f' {safe_windows(str(extra_argument))}'

    script: str = '\n'.join([
        # disable echoing commands to the command line
        '@echo off',

        f'chcp {ctypes.cdll.kernel32.GetACP()}',

        # "call" is needed to avoid the batch script from closing after running the first (environment activation) line
        f'call {safe_windows(activate)} {safe_windows(prefix)}',

        f'{command} >{safe_windows(stdout_log_path)} 2>{safe_windows(stderr_log_path)}',

        '',
    ])

    file_path: str = create_app_run_script(
        root_prefix=root_prefix,
        prefix=prefix,
        command=script,
        extension='.bat',
        package_name=package_name,
        default_scripts_path=default_scripts_path,
    )
    return {  # popen_dict
        'creationflags': subprocess.CREATE_NO_WINDOW | subprocess.CREATE_NEW_PROCESS_GROUP,  # type: ignore
        'cwd': cwd,
        'env': environment,
    }, {
        'args': file_path,
        'id': id_,
        'cmd': script,
        'stdout': stdout_log_path,
        'stderr': stderr_log_path,
    }


def get_command_on_unix(  # pylint: disable=too-many-arguments
        root_prefix: str,
        prefix: str,
        command: str,
        extra_arguments: typing.Iterable[typing.Any] = (),
        package_name: str = 'app',
        environment: typing.Optional[typing.Mapping[str, str]] = None,
        cwd: str = os.path.expanduser('~'),
        default_scripts_path: str = LAUNCH_SCRIPTS_PATH,
) -> typing.Tuple[typing.Mapping[str, typing.Any], 'CommandDetails']:
    """
    Generate script to launch application and return path to it.

    This function is optimized to run on Linux/OS X.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param command: Actual command to launch application.
    :param extra_arguments: Additional arguments to attach to command.
    :param cwd: Custom working directory to launch application in.
    :param package_name: Name of the conda package, or alias of the external application.
    :param environment: Custom environment to launch application in.
    :param default_scripts_path: Root directory to store launch scripts in.
    :return: Path to generated launch script file.
    """
    stdout_log_path: str
    stderr_log_path: str
    id_: int
    stdout_log_path, stderr_log_path, id_ = get_package_logs(
        root_prefix=root_prefix,
        prefix=prefix,
        package_name=package_name,
        default_scripts_path=default_scripts_path,
    )

    command = expand_environment(
        command=command,
        environment={
            'PREFIX': prefix,
            'CONDA_PREFIX': prefix,
            'CONDA_ROOT_PREFIX': root_prefix,
        },
    )
    activate: str = os.path.join(root_prefix, 'bin', 'activate')

    extra_argument: typing.Any
    for extra_argument in extra_arguments:
        command += f' {safe_unix(str(extra_argument))}'

    script: str = '\n'.join([
        '#!/usr/bin/env bash',
        f'. {safe_unix(activate)} {safe_unix(prefix)}',
        f'{command} >{safe_unix(stdout_log_path)} 2>{safe_unix(stderr_log_path)}',
        ''
    ])

    file_path: str = create_app_run_script(
        root_prefix=root_prefix,
        prefix=prefix,
        command=script,
        extension='.sh',
        package_name=package_name,
        default_scripts_path=default_scripts_path
    )
    return {
        'cwd': cwd,
        'env': environment,
    }, {
        'args': file_path,
        'id': id_,
        'cmd': script,
        'stdout': stdout_log_path,
        'stderr': stderr_log_path,
    }


def launch(  # pylint: disable=too-many-arguments
        root_prefix: str,
        prefix: str,
        command: str,
        extra_arguments: typing.Iterable[typing.Any] = (),
        working_directory: str = os.path.expanduser('~'),
        package_name: str = 'app',
        environment: typing.Optional[typing.Mapping[str, str]] = None,
        leave_path_alone: bool = True,  # pylint: disable=unused-argument
        non_conda: bool = False,  # pylint: disable=unused-argument
) -> typing.Optional[RunningProcess]:
    """
    Handle launching commands from projects.

    :param root_prefix: Path to root (base) conda prefix. Used to activate `prefix`.
    :param prefix: Conda prefix, which should be active.
    :param command: Actual command to launch application.
    :param extra_arguments: Additional arguments to attach to command.
    :param working_directory: Custom working directory to launch application in.

                              If not provided - home directory will be used.
    :param package_name: Name of the conda package, or alias of the external application.
    :param environment: Custom environment to launch application in.
    :param as_admin: Launch application with admin rights.

                     This breaks function result (changes it to :code:`True`/:code:`None`).
    :return: Description of the launched process.
    """
    if isinstance(extra_arguments, str):
        extra_arguments = (extra_arguments,)

    get_command: 'GetCommand' = get_command_on_unix
    if WIN:
        get_command = get_command_on_win

    popen_dict: typing.Mapping[str, typing.Any]
    extra_args: 'CommandDetails'
    popen_dict, extra_args = get_command(
        root_prefix=root_prefix,
        prefix=prefix,
        command=command,
        extra_arguments=extra_arguments,
        package_name=package_name,
        environment=environment,
        cwd=working_directory,
    )

    args: str = extra_args['args']
    if WIN:
        args = f'cmd.exe /C {safe_windows(args)}'

    return RunningProcess(
        package=package_name,
        process=subprocess.Popen(args, **popen_dict),  # nosec
        stdout=extra_args['stdout'],
        stderr=extra_args['stderr'],
    )



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright 2016 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""This folder contains css for QSS styling of the application."""

from __future__ import absolute_import, division, print_function

# Standard library imports
import os.path as osp

DATA_PATH = osp.dirname(osp.realpath(__file__))
GLOBAL_STYLES_PATH = osp.join(DATA_PATH, 'styles.css')



# -*- coding: utf-8 -*-

# pylint: disable=invalid-name

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Styles for the application."""

import os
from navigator_updater.static import images
from navigator_updater.static.css import GLOBAL_STYLES_PATH


def load_style_sheet():
    """Load css styles file and parse to include custom variables."""
    with open(GLOBAL_STYLES_PATH, 'rt', encoding='utf-8') as f:
        data = f.read()

    if os.name == 'nt':
        data = data.replace('$IMAGE_PATH', images.IMAGE_PATH.replace('\\', '/'))
    else:
        data = data.replace('$IMAGE_PATH', images.IMAGE_PATH)

    return data



# -*- coding: utf-8 -*-

"""Collection of utility components to use by APIs."""

from __future__ import annotations

__all__ = ['split_canonical_name']

import typing


def split_canonical_name(cname: str) -> typing.Tuple[str, ...]:
    """Split a canonical package name into name, version, build."""
    return tuple(cname.rsplit('-', 2))



#  tests for navigator-updater-0.4.0-py310h06a4308_0 (this is a generated file);
print('===== testing package: navigator-updater-0.4.0-py310h06a4308_0 =====');
print('running run_test.py');
#  --- run_test.py (begin) ---
import navigator_updater

print('navigator_updater.__version__: %s' % navigator_updater.__version__)
assert navigator_updater.__version__ == '0.4.0'
#  --- run_test.py (end) ---

print('===== navigator-updater-0.4.0-py310h06a4308_0 OK =====');
print("import: 'navigator_updater'")
import navigator_updater




# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright 2016 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Module in charge of the configuration settings."""

# Standard library imports
import os
import sys

# Local imports
from navigator_updater import __version__
from navigator_updater.config.base import get_conf_path, get_home_dir

# FLAGS
TEST_CI = os.environ.get('TEST_CI', False)
MAC = sys.platform == 'darwin'
WIN = os.name == 'nt'
LINUX = sys.platform.startswith('linux')
DEV = 'dev' in __version__

HOME_PATH = get_home_dir()
CONF_PATH = get_conf_path()
LAUNCH_SCRIPTS_PATH = os.path.join(CONF_PATH, 'scripts')
LOCKFILE = os.path.join(CONF_PATH, 'navigator_updater.lock')
NAVIGATOR_LOCKFILE = os.path.join(CONF_PATH, 'navigator.lock')
PIDFILE = os.path.join(CONF_PATH, 'navigator.pid')

# Logging
LOG_FOLDER = os.path.join(CONF_PATH, 'logs')
LOG_FILENAME = 'navigator_updater.log'
MAX_LOG_FILE_SIZE = 2 * 1024 * 1024



#!/usr/bin/env python

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright 2016 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Application start."""

# Standard library imports
import signal
import sys

# Third party imports
from qtpy import PYQT5
from qtpy.QtCore import QCoreApplication, QEvent, QObject, Qt
from qtpy.QtGui import QIcon

# Local imports
from navigator_updater.config import LOCKFILE, MAC
from navigator_updater.external import filelock
from navigator_updater.static import images
from navigator_updater.static.fonts import load_fonts
from navigator_updater.utils.logs import setup_logger, LOGGER_CONFIG
from navigator_updater.utils.qthelpers import qapplication
from navigator_updater.widgets.dialogs import MessageBoxInformation
from navigator_updater.widgets.dialogs.main_dialog import MainDialog

# For retina displays on qt5
if hasattr(Qt, 'AA_UseHighDpiPixmaps'):
    QCoreApplication.setAttribute(Qt.AA_UseHighDpiPixmaps)

if hasattr(Qt, 'AA_EnableHighDpiScaling'):
    QCoreApplication.setAttribute(Qt.AA_EnableHighDpiScaling)


def except_hook(cls, exception, traceback):
    """Custom except hook to avoid crashes on PyQt5."""
    sys.__excepthook__(cls, exception, traceback)


def set_application_icon():
    """Set application icon."""
    global app  # pylint: disable=global-variable-not-assigned,invalid-name
    app_icon = QIcon(images.ANACONDA_LOGO)
    app.setWindowIcon(app_icon)


class EventEater(QObject):  # pylint: disable=too-few-public-methods
    """Event filter for application state."""

    def __init__(self, application):
        """Event filter for application state."""
        super().__init__()
        self.app = application

    def eventFilter(self, ob, event):  # pylint: disable=invalid-name
        """Qt override."""
        if (event.type() == QEvent.ApplicationActivate) and MAC and self.app.window.setup_ready:
            self.app.window.show()
            if self.app.window.isMaximized():
                self.app.window.showMaximized()
            elif self.app.window.isFullScreen():
                self.app.window.showFullScreen()
            else:
                self.app.window.showNormal()
            return True

        return super().eventFilter(ob, event)


def start_app(options):  # pragma: no cover
    """Main application entry point."""
    # Setup logger
    LOGGER_CONFIG.level = options.log_level
    setup_logger()

    # Monkey patching sys.excepthook to avoid crashes in PyQt 5.5+
    if PYQT5:
        sys.excepthook = except_hook

    global app  # pylint: disable=invalid-name,global-variable-undefined,global-statement
    app = qapplication(test_time=60)
    set_application_icon()
    load_fonts(app)
    signal.signal(signal.SIGINT, signal.SIG_DFL)

    # Create file lock
    lock = filelock.FileLock(LOCKFILE)
    try:
        with lock.acquire(timeout=1.0):
            dlg = MainDialog(
                latest_version=options.latest_version, prefix=options.prefix
            )
            app.window = dlg
            event_eater = EventEater(app)
            app.installEventFilter(event_eater)
            sys.exit(dlg.exec_())
    except filelock.Timeout:
        msgbox = MessageBoxInformation(
            title='Anaconda Navigator Updater Information',
            text='There is an instance of Anaconda Navigator Updater already running.',
        )
        sys.exit(msgbox.exec_())



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Base configuration management."""

import contextlib
import os
import typing


# -----------------------------------------------------------------------------
# --- Configuration paths
# -----------------------------------------------------------------------------

SUBFOLDER: typing.Final[str] = os.path.join('.anaconda', 'navigator')


def get_home_dir() -> str:
    """Return user home directory."""
    if os.name == 'nt':
        fallback: str = ''
        path_env_var: str
        path_env_vars: typing.Sequence[str] = ('HOME', 'APPDATA', 'USERPROFILE', 'TMP')

        # look through all available values
        for path_env_var in path_env_vars:
            current: str = os.environ.get(path_env_var, '')
            if (not current) or (not os.path.isdir(current)):
                continue

            # save first existing folder as a fallback value
            fallback = fallback or current

            # prefer options which already contain `.anaconda\navigator`
            inner: str = os.path.join(current, SUBFOLDER)
            if os.path.isdir(inner):
                return current

        # if there was any working option - use it
        if fallback:
            return fallback

    else:
        with contextlib.suppress(BaseException):
            return os.path.expanduser('~')

    raise RuntimeError('Please define environment variable $HOME')


def get_conf_path(filename: typing.Optional[str] = None) -> str:
    """Return absolute path for configuration file with specified filename."""
    result: str = os.path.join(get_home_dir(), SUBFOLDER)
    os.makedirs(result, exist_ok=True)

    if filename is not None:
        return os.path.join(result, filename)
    return result



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) Spyder Project Contributors

# Licensed under the terms of the MIT License
# (see spyder/__init__.py for details)
# -----------------------------------------------------------------------------

"""
Text encoding utilities, text file I/O.

Functions 'get_coding', 'decode' and 'encode' come from Eric4 source code (Utilities/__init___.py)

Copyright © 2003-2009 Detlev Offenbach
"""

import locale
import sys
from navigator_updater.utils.py3compat import is_binary_string, is_string, is_unicode, to_text_string


PREFERRED_ENCODING = locale.getpreferredencoding()

# -----------------------------------------------------------------------------
#  Functions for encoding and decoding bytes that come from
#  the *file system*.
# -----------------------------------------------------------------------------


# The default encoding for file paths and environment variables should be set to match the default encoding that the OS
# is using.
def getfilesystemencoding():
    """
    Query the filesystem for the encoding used to encode filenames and envvars.
    """
    encoding = sys.getfilesystemencoding()
    if encoding is None:
        # Must be Linux or Unix and nl_langinfo(CODESET) failed.
        encoding = PREFERRED_ENCODING
    return encoding


FS_ENCODING = getfilesystemencoding()


def to_unicode_from_fs(string):
    """
    Return a unicode version of string decoded using the file system encoding.
    """
    if not is_string(string):  # string is a QString
        string = to_text_string(string.toUtf8(), 'utf-8')
    else:
        if is_binary_string(string):
            try:
                unic = string.decode(FS_ENCODING)
            except (UnicodeError, TypeError):
                pass
            else:
                return unic
    return string


def to_fs_from_unicode(unic):
    """
    Return a byte string version of unc encoded using the file system encoding.
    """
    if is_unicode(unic):
        try:
            string = unic.encode(FS_ENCODING)
        except (UnicodeError, TypeError):
            pass
        else:
            return string
    return unic


def ensure_binary(value):
    """Ensure a file is binary format. From conda/common."""
    try:
        return value.encode('utf-8')
    except AttributeError:
        # AttributeError: '<>' object has no attribute 'encode'
        # In this case assume already binary type and do nothing
        return value



""" Logging configs and extensions. """

import logging.config
import sys
import threading
import types
import typing

from navigator_updater.utils.logs.common import *
from navigator_updater.utils.logs.loggers import *
from navigator_updater.utils.logs.config import LOGGER_CONFIG


def global_exception_logger(
        exc_type: typing.Type[BaseException],
        exception: BaseException,
        traceback: typing.Optional[types.TracebackType],
) -> None:
    """Handle an exception by logs it."""
    logger.exception(exception, exc_info=(exc_type, exception, traceback))


def setup_logger() -> None:
    """Setup, create, and set logger."""
    logging.config.dictConfig(LOGGER_CONFIG.dict_config)

    # Note: threading.excepthook is only supported since Python 3.8
    sys.excepthook = global_exception_logger
    sys.unraisablehook = global_exception_logger  # type: ignore
    threading.excepthook = global_exception_logger  # type: ignore

    logger.debug('Setting up logger')



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Logger utilities."""

__all__ = ['log_files', 'clean_logs', 'load_log', 'JSONFormatter']

import datetime
import logging.handlers
import os
import types
import typing

import ujson

from navigator_updater.config import LOG_FILENAME, LOG_FOLDER

LOG_TIME_FORMAT: str = '%Y-%m-%d %H:%M:%S'

_SysExcInfoType = typing.Union[
    typing.Tuple[typing.Type[BaseException], BaseException, typing.Optional[types.TracebackType]],
    typing.Tuple[None, None, None]
]

_LogRecordAttrType = typing.Union[int, float, str, _SysExcInfoType, None]


class JSONFormatter(logging.Formatter):
    """
    Simple logs formatter using JSON serialization.
    """

    def __init__(
            self,
            fields: typing.Optional[typing.Dict[str, str]] = None,
            always_extra: typing.Optional[typing.Dict[str, str]] = None,
            datefmt: typing.Optional[str] = LOG_TIME_FORMAT,
    ) -> None:
        """
        :param fields: A dictionary of fields to use in the log.
            The keys in the dictionary are keys that will be used in the
            final log form, and its values are the names of the attributes
            from the log record to use as final log values. Defaults to
            None, which is interpreted as an empty dict.
        :param always_extra: A dictionary of additional static
            values written to the final log. Defaults to None, which is
            interpreted as an empty dict.
        :param datefmt: strftime date format. For more details
            check logs.Formatter documentation. Defaults to None.
        """
        super().__init__(fmt=None, datefmt=datefmt, style='%')
        self.fields: typing.Dict[str, str] = fields or {}
        self._uses_time: bool = 'asctime' in self.fields.values()
        self.always_extra: typing.Dict[str, typing.Any] = always_extra or {}

    def usesTime(self) -> bool:
        """
        Check if the format uses the creation time of the record. For more
        information about the method see logs.Formatter.
        """
        return self._uses_time

    def format(self, record: logging.LogRecord) -> str:
        """
        Build a JSON serializable dict starting from `self.always_extra`,
        adding the data from the LogRecord specified in `self.fields`, and
        finally adding the record specific extra data.

        :param record: log record to be converted to string
        :returns: JSON serialized log record
        """
        data: typing.MutableMapping[str, typing.Any] = self.always_extra.copy()
        record.message = record.getMessage()

        if self.usesTime():
            record.asctime = self.formatTime(record, self.datefmt)

        if record.exc_info and not record.exc_text:
            record.exc_text = self.formatException(record.exc_info)

        if record.stack_info:
            record.stack_info = self.formatStack(record.stack_info)

        key: str
        field: str
        for key, field in self.fields.items():
            value: _LogRecordAttrType = record.__dict__.get(field, None)

            if not value:
                continue

            if field == 'exc_text':
                value = record.exc_text if any(record.exc_info or []) else None

            data[key] = value

        return ujson.dumps(data)  # pylint: disable=c-extension-no-member


def log_files(log_folder: str = LOG_FOLDER, log_filename: str = LOG_FILENAME) -> typing.List[str]:
    """
    Return all available log files located inside the logs folder.

    Files starting with a `.` are ignored as well as files not including the
    `log_filename` as part of the name.
    """
    paths: typing.List[str] = []
    if not os.path.isdir(log_folder):
        return paths

    log_file: str
    for log_file in sorted(os.listdir(log_folder)):
        log_file_path: str = os.path.join(log_folder, log_file)

        if os.path.isfile(log_file_path) and (log_filename in log_file) and (not log_file.startswith('.')):
            paths.append(log_file_path)

    return paths


def clean_logs(log_folder: str = LOG_FOLDER) -> None:
    """Remove logs in old plain text format."""
    week_ago: datetime.datetime = datetime.datetime.now() - datetime.timedelta(days=7)
    to_datetime: typing.Callable[[str, str], datetime.datetime] = datetime.datetime.strptime

    path: str
    lines: typing.List[typing.Mapping[str, typing.Any]]
    for path in log_files(log_folder):
        lines = load_log(path)

        line: typing.Any
        new_lines: typing.List[str] = []
        for line in lines:
            try:
                if to_datetime(line['time'], LOG_TIME_FORMAT) < week_ago:
                    continue
            except (ValueError, TypeError, KeyError):
                pass
            new_lines.append(ujson.dumps(line, ensure_ascii=False) + '\n')  # pylint: disable=c-extension-no-member

        with open(path, 'w', encoding='utf-8') as stream:
            stream.writelines(new_lines)


def load_log(log_file_path: str) -> typing.List[typing.Mapping[str, typing.Any]]:
    """Load log file and return list of items."""
    if not os.path.isfile(log_file_path):
        return []

    json_lines: typing.List[typing.Mapping[str, typing.Any]] = []
    with open(log_file_path, 'r', encoding='utf-8') as stream:
        for line in stream:
            try:
                json_lines.append(ujson.loads(line))  # pylint: disable=c-extension-no-member
            except ValueError:
                pass

    return json_lines



"""Logging configurations."""

__all__ = ('LOGGER_CONFIG',)

from dataclasses import dataclass
import logging
import os

from navigator_updater.config import LOG_FILENAME, LOG_FOLDER, MAX_LOG_FILE_SIZE

if not os.path.isdir(LOG_FOLDER):
    os.makedirs(LOG_FOLDER)


@dataclass
class LoggerConfig:
    """Config variables and logging dictionary configuration. """
    file_path: str = os.path.join(LOG_FOLDER, LOG_FILENAME)
    backup_count: int = 5
    level: int = logging.INFO
    max_log_file_size = MAX_LOG_FILE_SIZE

    @property
    def dict_config(self):
        """Logging dict config."""
        return {
            'version': 1,
            'formatters': {
                'root_console': {
                    'format': '%(asctime)s - %(levelname)s %(module)s.%(funcName)s:%(lineno)d\n%(message)s\n'
                },
                'root_json': {
                    '()': 'navigator_updater.utils.logs.common.JSONFormatter',
                    'fields': {
                        'time': 'asctime',
                        'level': 'levelname',
                        'module': 'module',
                        'method': 'funcName',
                        'line': 'lineno',
                        'path': 'pathname',
                        'message': 'message',
                        'exception': 'exc_text',
                        'traceback': 'stack_info'
                    }
                },
                'conda_json': {
                    '()': 'navigator_updater.utils.logs.common.JSONFormatter',
                    'fields': {
                        'time': 'asctime',
                        'level': 'levelname',
                        'module': 'module',
                        'method': 'funcName',
                        'line': 'lineno',
                        'path': 'pathname',
                        'message': 'message',
                        'output': 'output',
                        'error': 'error',
                        'environment': 'environment',
                        'callback': 'callback',
                        'exception': 'exc_text',
                        'traceback': 'stack_info'
                    }
                },
                'http_json': {
                    '()': 'navigator_updater.utils.logs.common.JSONFormatter',
                    'fields': {
                        'time': 'asctime',
                        'level': 'levelname',
                        'module': 'module',
                        'method': 'funcName',
                        'line': 'lineno',
                        'path': 'pathname',
                        'message': 'message',
                        'exception': 'exc_text',
                        'traceback': 'stack_info',
                        'request_method': 'request_method',
                        'request_url': 'request_url',
                        'request_body': 'request_body',
                        'request_headers': 'request_headers',
                        'response_code': 'response_code',
                        'response_headers': 'response_headers',
                        'response_body': 'response_body',
                    }
                },
            },
            'handlers': {
                'console': {
                    'class': 'logging.StreamHandler',
                    'formatter': 'root_console',
                    'level': LOGGER_CONFIG.level if LOGGER_CONFIG.level <= logging.DEBUG else max(
                        logging.WARNING, LOGGER_CONFIG.level),
                },
                'file': {
                    'class': 'logging.handlers.RotatingFileHandler',
                    'formatter': 'root_json',
                    'level': LOGGER_CONFIG.level,
                    'maxBytes': LOGGER_CONFIG.max_log_file_size,
                    'filename': LOGGER_CONFIG.file_path,
                    'backupCount': LOGGER_CONFIG.backup_count
                },
                'conda_file': {
                    'class': 'logging.handlers.RotatingFileHandler',
                    'formatter': 'conda_json',
                    'level': LOGGER_CONFIG.level,
                    'maxBytes': LOGGER_CONFIG.max_log_file_size,
                    'filename': LOGGER_CONFIG.file_path,
                    'backupCount': LOGGER_CONFIG.backup_count
                },
                'http_file': {
                    'class': 'logging.handlers.RotatingFileHandler',
                    'formatter': 'http_json',
                    'level': LOGGER_CONFIG.level,
                    'maxBytes': LOGGER_CONFIG.max_log_file_size,
                    'filename': LOGGER_CONFIG.file_path,
                    'backupCount': LOGGER_CONFIG.backup_count
                }
            },
            'loggers': {
                '': {
                    'level': LOGGER_CONFIG.level,
                    'handlers': ['console', 'file']
                },
                'navigator_updater.conda': {
                    'level': logging.DEBUG,
                    'handlers': ['console', 'conda_file']
                },
                'navigator_updater.http': {
                    'level': logging.DEBUG,
                    'handlers': ['console', 'http_file']
                },
            }
        }


LOGGER_CONFIG = LoggerConfig()



# -*- coding: utf-8 -*-

# -----------------------------------------------------------------------------
# Copyright 2016 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------

"""Command line arguments."""

# Standard library imports
import argparse
import logging


def parse_arguments():
    """Parse CLI arguments."""
    parser = argparse.ArgumentParser(description='Navigator Updater.')
    parser.add_argument(
        '--version',
        help='Print version information',
        action='store_const',
        const=True,
        default=False,
        dest='version',
    )
    parser.add_argument(
        '--latest-version',
        help='Latest version available for update',
        action='store',
        dest='latest_version',
        const=None,
        default=None,
        nargs='?',
    )
    parser.add_argument(
        '--prefix',
        help='Environment prefix on which to perform the update',
        action='store',
        dest='prefix',
        const=None,
        default=None,
        nargs='?',
    )
    parser.add_argument(
        '--debug',
        help='Print debug statements',
        action='store_const',
        dest='log_level',
        const=logging.DEBUG,
        default=logging.WARNING,
    )

    return parser.parse_args()
